require "util"
require "defines"
require "story"

game.on_init(function()
  game.player.force.disable_all_prototypes()
  game.map_settings.enemy_expansion.enabled = false
  game.evolution_factor = 0.45
  game.map_settings.enemy_evolution.enabled = false
  game.always_day = true

  local recipelist = game.player.force.recipes
  recipelist["iron-plate"].enabled = true
  recipelist["copper-plate"].enabled = true
  recipelist["stone-brick"].enabled = true
  recipelist["stone-wall"].enabled = true
  recipelist["wood"].enabled = true
  recipelist["stone-furnace"].enabled = true
  recipelist["iron-stick"].enabled = true
  recipelist["iron-axe"].enabled = true
  recipelist["wooden-chest"].enabled = true
  recipelist["iron-gear-wheel"].enabled = true
  recipelist["basic-transport-belt"].enabled = true
  recipelist["electronic-circuit"].enabled = true
  recipelist["copper-cable"].enabled = true
  recipelist["pistol"].enabled = true
  recipelist["submachine-gun"].enabled = true
  recipelist["basic-bullet-magazine"].enabled = true
  recipelist["basic-mining-drill"].enabled = true
  recipelist["basic-inserter"].enabled = true
  recipelist["small-electric-pole"].enabled = true
  recipelist["solar-panel"].enabled = true
  recipelist["repair-pack"].enabled = true

  local technologylist = game.player.force.technologies
  technologylist["steel-processing"].researched = true
  technologylist["optics"].researched = true
  technologylist["logistics"].researched = true
  technologylist["logistics-2"].researched = true
  technologylist["automation"].researched = true
  technologylist["automation-2"].researched = true
  technologylist["flame-thrower"].enabled = true
  technologylist["electronics"].researched = true
  technologylist["advanced-electronics"].researched = true
  technologylist["military"].researched = true
  technologylist["military-2"].researched = true
  technologylist["turrets"].researched = true
  technologylist["armor-making"].researched = true
  technologylist["bullet-damage-1"].researched = true
  technologylist["bullet-damage-2"].researched = true
  technologylist["bullet-speed-1"].researched = true
  technologylist["bullet-speed-2"].researched = true

  local character = game.player.character
  character.insert{name = "basic-grenade", count = 7}
  character.insert{name = "fast-transport-belt", count = 128}
  character.insert{name = "fast-inserter", count = 45}
  character.insert{name = "medium-electric-pole", count = 40}
  character.insert{name = "long-handed-inserter", count = 30}
  character.insert{name = "assembling-machine-2", count = 20}
  character.insert{name = "stone-wall", count = 128}
  character.insert{name = "gun-turret", count = 15}
  character.insert{name = "steel-chest", count = 5}
  character.insert{name = "coal", count = 200}
  character.insert{name = "submachine-gun", count = 1}
  character.insert{name = "piercing-bullet-magazine", count = 100}
  character.insert{name = "steel-axe", count = 1}
  character.insert{name = "heavy-armor", count = 1}
  character.insert{name = "solar-panel", count = 50}
  character.insert{name = "iron-plate", count = 500}
  character.insert{name = "copper-plate", count = 500}

  local car = game.get_entity_by_tag("car")
  car.insert{name = "piercing-bullet-magazine", count = 2000}
  car.insert{name = "iron-plate", count = 1500}
  car.insert{name = "copper-plate", count = 1500}
  character.insert{name = "steel-plate", count = 1000}
  car.insert{name = "coal", count = 1000}

  local techwreckage = game.get_entity_by_tag("tech-wreckage")
  techwreckage.insert{name = "explosives", count = 50}
  techwreckage.insert{name = "defender-capsule", count = 10}
  techwreckage.insert{name = "slowdown-capsule", count = 10}

  local bonuswreckage = game.get_entity_by_tag("bonus-wreckage")
  bonuswreckage.insert{name = "defender-capsule", count = 25}
  bonuswreckage.insert{name = "rocket", count = 25}
end)

storytable =
{
  {
    {
      condition = story.Story:elapsedcheck(3),
      action =
      function()
        game.player.print({"msg-wreck-location"})
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      action =
      function()
        game.player.print({"msg-wreck-motivation"})
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      action =
      function()
        game.player.print({"msg-wreck-guarded"})
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      action =
      function()
        game.player.print({"msg-wrech-technology"})
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      action = function() game.show_message_dialog{text = {"msg-grenade"}} end
    },
    {
      condition = story.Story:elapsedcheck(2),
      init =
      function(event)
        game.player.set_goal_description({"goal-get-wreckage"})
        game.player.set_gui_arrow({type="entity", entity=game.get_entity_by_tag("tech-wreckage")})
      end,
    },
    {
      condition =
      function()
        local wreck = game.get_entity_by_tag("tech-wreckage")
        return game.get_entity_by_tag("tech-wreckage") == game.player.opened or
               not wreck.has_items_inside()
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      init =
      function(event)
        game.player.clear_gui_arrow()
        game.player.set_goal_description("")
        game.show_message_dialog{text = {"msg-could-make-more-1"}}
        game.show_message_dialog{text = {"msg-could-make-more-2"}}
        game.show_message_dialog{text = {"msg-could-make-more-3"}}
        local technologies = game.player.force.technologies
        technologies["battery"].researched = true
        technologies["rocketry"].researched = true
        technologies["combat-robotics"].researched = true
        technologies["follower-robot-count-1"].researched = true
        technologies["follower-robot-count-2"].researched = true
        technologies["follower-robot-count-3"].researched = true
        technologies["follower-robot-count-4"].researched = true
        technologies["follower-robot-count-5"].researched = true
        technologies["follower-robot-count-6"].researched = true
      end
    },
    {
      condition = story.Story:elapsedcheck(5),
      init =
      function(event)
        game.player.set_goal_description({"goal-get-computer"})
      end
    },
    {
      condition = function() return game.player.get_item_count("computer") > 0 end,
      action = function()
               game.player.print({"msg-got-it"})
             end,
    },
    {
      condition = story.Story:elapsedcheck(5)
    }
  }
}

mainstory = story.Story:new(storytable)

game.on_save(function()
  global.storydata = mainstory:export()
end)

game.on_load(function()
  mainstory:import(global.storydata)
end)

function difficultynumber(easy, normal, hard)
  if game.difficulty == defines.difficulty.easy then
    return easy
  elseif game.difficulty == defines.difficulty.normal then
    return normal
  else
    return hard
  end
end

function manageattacks(tick)
  if global.lastattackat == nil then
    global.lastattackat = difficultynumber(120, 60, 0)
  end
  if global.attack_count == nil then
    global.attack_count = 3
  end
  -- attack rate is once per 90 seconds, it doesn't change
  if global.attackfrequency == nil then
    global.attackfrequency = difficultynumber(50, 40, 30)
  end
  if tick - global.lastattackat > 60 * global.attackfrequency then
    global.lastattackat = tick
    game.player.surface.set_multi_command({type=defines.command.attack,
                                          target=game.player.character,
                                          distraction=defines.distraction.by_enemy},
                                          global.attack_count)
    -- every attack is 0.6 aliens bigger
    global.attack_count = global.attack_count + 0.7
  end
end

game.on_event(defines.events, function(event)
  manageattacks(event.tick)
  mainstory:update(event, "level-04")
end)
